/***************************************************************************
                          confdialog.cpp  -  description
                             -------------------
    begin                : thu feb 20 2003
    copyright            : (C) 2003 by Ren-Pierre Lehmann
    email                : ripi@lepi.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qcombobox.h>
#include <qwhatsthis.h>
#include <qgroupbox.h>
#include <qtooltip.h>
#include <qspinbox.h>

#include <klocale.h>
#include <kconfig.h>

#include "confdialog.h"
 
ConfDialog::ConfDialog( QWidget *parent )
    : KDialogBase(  Plain, i18n( "Kickarts Preferences" ),
          Ok|Cancel|Apply, Ok, parent )
{

    page = plainPage();

    QBoxLayout *layout = new QVBoxLayout( page );
    layout->setSpacing( KDialog::spacingHint() );

    verboseChk = new QCheckBox( i18n("&Notify when an error occurs"), page );
    layout->addWidget( verboseChk );
    
    QHBox *loopTimeGroup = new QHBox(page);
    (void) new QLabel(i18n("Update interval: "), loopTimeGroup);
    loopTimeBox = new QSpinBox( 0, 3600, 1, loopTimeGroup);
    QToolTip::add(loopTimeGroup, i18n("A value of 0 disables updates") );
    (void) new QLabel(i18n(" seconds"), loopTimeGroup);
    layout->addWidget(loopTimeGroup);
    
    QGroupBox *buttonGroup = new QGroupBox(2, Vertical, i18n("Button Action"), page);
    layout->addWidget(buttonGroup);

    QHBox *buttonGroupBox = new QHBox(buttonGroup);
    QVBox *buttonGroupLeft = new QVBox(buttonGroupBox);
    QVBox *buttonGroupRight = new QVBox(buttonGroupBox);    

    QStringList buttonItems;
    buttonItems << i18n("Start sound server")
                << i18n("Suspend sound server")
                << i18n("Terminate sound server")
                << i18n("Restart sound server")
                << i18n("Display status information")
                << i18n("Show menu");

    leftAction = new QComboBox(buttonGroupRight);
    leftAction->insertStringList(buttonItems);
    (void) new QLabel(leftAction, i18n("&Left Button:"), buttonGroupLeft);

    QHBox *rightButton = new QHBox(buttonGroupRight);
    rightAction = new QComboBox(rightButton);
    rightAction->insertStringList(buttonItems);
    (void) new QLabel(rightAction, i18n("&Right Button:"), buttonGroupLeft);

    QHBox *midButton = new QHBox(buttonGroupRight);
    midAction = new QComboBox(midButton);
    midAction->insertStringList(buttonItems);
    (void) new QLabel(midAction, i18n("&Middle Button:"), buttonGroupLeft);

    layout->addStretch();

    connect( this, SIGNAL(applyClicked()), this, SLOT(apply()) );
    connect( this, SIGNAL(okClicked()), this, SLOT(apply()) );
}

ConfDialog::~ConfDialog()
{
    // no need to delete child widgets, qt does it all for us
}

int ConfDialog::loopTime() const {
    return loopTimeBox->value();
}

bool ConfDialog::verboseMode() const {
    return verboseChk->isChecked();
}

int ConfDialog::leftButtonAction() const {
    return leftAction->currentItem();
}

int ConfDialog::rightButtonAction() const {
    return rightAction->currentItem();
}

int ConfDialog::midButtonAction() const {
    return midAction->currentItem();
}

void ConfDialog::setLoopTime( int t ){
    loopTimeBox->setValue(t);
}

void ConfDialog::setVerboseMode( bool b ){
    verboseChk->setChecked(b);
}

void ConfDialog::setLeftButtonAction( int i ){
    leftAction->setCurrentItem(i);
}

void ConfDialog::setRightButtonAction( int i ){
    rightAction->setCurrentItem(i);
}

void ConfDialog::setMidButtonAction( int i ){
    midAction->setCurrentItem(i);
}

#include "confdialog.moc"
