/***************************************************************************
                          kickarts.cpp  -  description
                             -------------------
    begin                : wed feb 19 2003
    copyright            : (C) 2003 by Ren-Pierre Lehmann
    email                : ripi@lepi.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h> 
  
#include <qlabel.h>
#include <qpainter.h>
#include <qtooltip.h>
#include <qcursor.h>
#include <qprocess.h>
#include <qdir.h>
#include <qfile.h>
#include <qtimer.h>
#include <qlayout.h>

#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kdialogbase.h>
#include <khelpmenu.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kseparator.h>
#include <kpopupmenu.h>
#include <kprocess.h>
#include <ksimpleconfig.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kpassivepopup.h>
#include <kstandarddirs.h>

#include <dispatcher.h>
#include <soundserver.h>

#include "kickarts.h"
#include "confdialog.h"

extern "C"
{
    KPanelApplet * init(QWidget * parent, const QString & configFile)
    {
        KGlobal::locale()->insertCatalogue("kickarts");
        return new kickarts(configFile, KPanelApplet::Normal, 0, parent, "kickarts");
    }
}

kickarts::kickarts(const QString& configFile, Type type, int actions,
                         QWidget *parent, const char *name)
    : KPanelApplet(configFile, type, actions, parent, name)
{
    setBackgroundMode(QWidget::X11ParentRelative);

    readConfig();
    
    stoppedPixmap = QPixmap(locate("data", "kickarts/pics/stopped.png"));
    runningPixmap = QPixmap(locate("data", "kickarts/pics/running.png"));
    suspendedPixmap = QPixmap(locate("data", "kickarts/pics/suspended.png"));

    init_pm();
    adjustSize();

    configArts_process = 0;
    mConfigDlg = 0L;
    
    setToolTip(i18n("Kickarts - aRts control applet"));

    if ( loopTime > 0 )
    {
        artsStatusTimer= new QTimer(this);
        connect(artsStatusTimer, SIGNAL(timeout()), this, SLOT(repaint()));
        artsStatusTimer->start(loopTime * 1000);
    }
}

kickarts::~kickarts()
{
    delete artsStatusTimer;
}

void kickarts::about()
{
    kdDebug() << "telling about" << endl;
    KAboutData aboutData("Kickarts", "Kickarts", VERSION,
            I18N_NOOP("aRts control applet for the Kicker"), KAboutData::License_GPL_V2);
    aboutData.addAuthor("René-Pierre Lehmann", "", "ripi@lepi.org", "http://ripi.net/software/kickarts/");

    KAboutApplication about(&aboutData, this, 0);
//    about.setImage(locate("data", "kickarts.png"));
    about.exec();
}

void kickarts::configKickarts()
{
    readConfig();

    if (mConfigDlg)
    {
       mConfigDlg->show();
       mConfigDlg->raise();
    }
    else
    {
        mConfigDlg = new ConfDialog(this);

        mConfigDlg->setLoopTime(loopTime);
        mConfigDlg->setVerboseMode(bVerbose);
        mConfigDlg->setLeftButtonAction(leftButtonAction);
        mConfigDlg->setRightButtonAction(rightButtonAction);
        mConfigDlg->setMidButtonAction(midButtonAction);
        
        mConfigDlg->show();

        connect(mConfigDlg, SIGNAL(okClicked()), this, SLOT(applyConfig()));
        connect(mConfigDlg, SIGNAL(applyClicked()), this, SLOT(applyConfig()));
        connect(mConfigDlg, SIGNAL(finished()), this, SLOT(finishedConfig()));
    }
}

void kickarts::applyConfig()
{
    if( loopTime != mConfigDlg->loopTime() )
    {
        loopTime = mConfigDlg->loopTime();
        if( loopTime > 0 )
            artsStatusTimer->start(loopTime * 1000);
    }
    
    bVerbose =  mConfigDlg->verboseMode();
    leftButtonAction = mConfigDlg->leftButtonAction();   
    rightButtonAction = mConfigDlg->rightButtonAction();
    midButtonAction = mConfigDlg->midButtonAction();

    writeConfig();
}

void kickarts::finishedConfig()
{
    mConfigDlg->delayedDestruct();
    mConfigDlg = 0L;
}


void kickarts::readConfig()
{
    KConfig *kc = config();
    kc->setGroup("General");
    loopTime = kc->readNumEntry("LoopTime", 1);
    bVerbose = kc->readBoolEntry("VerboseMode", false);
    leftButtonAction = kc->readNumEntry("LeftButtonAction", 1);
    rightButtonAction = kc->readNumEntry("RightButtonAction", 5);
    midButtonAction = kc->readNumEntry("MidButtonAction", 4);
}

void kickarts::writeConfig()
{
    KConfig *kc = config();
    kc->setGroup("General");
    kc->writeEntry("LoopTime", loopTime);
    kc->writeEntry("VerboseMode", bVerbose);
    kc->writeEntry("LeftButtonAction", leftButtonAction);
    kc->writeEntry("RightButtonAction", rightButtonAction);    
    kc->writeEntry("MidButtonAction", midButtonAction);

    kc->sync();
}
    
int kickarts::widthForHeight(int) const
{
    return pm.width();
}

int kickarts::heightForWidth(int) const
{
    return pm.height();
}

void kickarts::init_pm()
{
    switch(status())
    {
        case -1:
            pm = stoppedPixmap;
            break;
        case 0:
            pm = suspendedPixmap;
            break;
        default:
            pm = runningPixmap;
    }
}

void kickarts::adjustSize()
{
    resize(pm.size());
}

void kickarts::mousePressEvent(QMouseEvent *event)
{
    int action;
    
    if (event->button() == LeftButton)
    {
        action = leftButtonAction;
    }
    else if (event->button() == RightButton)
    {
        action = rightButtonAction;
    }
    else if (event->button() == MidButton)
    {
        action = midButtonAction;
    }
    else
    {
        return;
    }

    switch (action)
    {
        case 0:
            start();
            break;
        case 1:
            suspend();
            break;
        case 2:
            terminate();
            break;
        case 3:
            restart();
            break;
        case 4:
            statusView();
            break;
        case 5:
            contextMenu();
            break;
    }
}

void kickarts::contextMenu()
{
        KPopupMenu* menu = new KPopupMenu(i18n("Kickarts"), this );
        KPopupMenu* helpmenu = new KPopupMenu( this );
        helpmenu->insertItem(i18n("Configure &sound server..."), this, SLOT(configArts()));
        helpmenu->insertItem(i18n("Configure &Kickarts..."), this, SLOT(configKickarts()));

        menu->insertItem(SmallIcon("player_play"), i18n("&Start sound server"), this, SLOT(start()));
        menu->insertItem(SmallIcon("player_pause"), i18n("S&uspend sound server"), this, SLOT(suspend()));
        menu->insertItem(SmallIcon("player_stop"), i18n("&Terminate sound server"), this, SLOT(terminate()));
        menu->insertItem(SmallIcon("reload"), i18n("&Restart sound server"), this, SLOT(restart()));
        menu->insertItem(SmallIcon("viewmag"), i18n("&Display status information"), this, SLOT(statusView()));
        menu->insertSeparator();
        menu->insertItem(i18n("&About"), this, SLOT(about()));
        menu->insertItem(SmallIcon("configure"), i18n("&Configure..."), helpmenu);

        menu->exec(QCursor::pos());
        delete menu;
        delete helpmenu;
}

void kickarts::paintEvent(QPaintEvent *)
{
    init_pm();

	QPainter p(this);
	int x = (width() - pm.width())/2;
	int y = (height() - pm.height())/2;
    if ( x < 0 ) x = 0;
    if ( y < 0 ) y = 0;
	p.drawPixmap(x , y, pm);
	p.end();
}

void kickarts::start()
{
    if(status() == -1)
    {
        start_arts();
    }
    else
    {
        if(bVerbose)
            KPassivePopup::message(i18n("Unable to start sound server"),
                i18n("Sound server is already running"), this);
    }
    repaint();
}

void kickarts::suspend()
{
    switch (status())
    {
        case -1:
            kdDebug() << "suspend: sound server not running, not suspending" << endl;
            if(bVerbose)
                KPassivePopup::message(i18n("Unable to suspend sound server"),
                    i18n("Sound server is not running"), this);
            break;

        case 0:
            kdDebug() << "suspend: sound server already suspended" << endl;
            if(bVerbose)
                KPassivePopup::message(i18n("Unable to suspend sound server"),
                    i18n("Sound server is already suspended"), this);
            break;

        case 2:
            kdDebug() << "suspend: sound server busy" << endl;
            if(bVerbose)
                KPassivePopup::message(i18n("Unable to suspend sound server"),
                    i18n("Sound server is busy"), this);
            break;

        default:
            if (server.suspend() == true)
            {
                kdDebug() << "suspend: sound server suspended"  << endl;
            } else {
                kdDebug() << "suspend: unable to suspend sound server" << endl;
                if(bVerbose)
                KPassivePopup::message(i18n("Unable to suspend sound server"),
                    i18n("Unknown reason"), this);
            }
    }

    repaint();
}

void kickarts::terminate()
{
    if(status() == -1)
    {
        kdDebug() << "terminate: sound server not running, not terminating" << endl;
        if(bVerbose)
            KPassivePopup::message(i18n("Unable to terminate sound server"),
                i18n("Sound server is not running"), this);
    }
    else
    {
        if(!server.terminate())
        {
            kdDebug() << "terminate: unable to terminate sound server" << endl;
            if(bVerbose)
                KPassivePopup::message(i18n("Unable to terminate sound server"),
                    i18n("Unknown reason"), this);
        }
    }
    repaint();
}

void kickarts::restart()
{
    if(status() == -1)
    {
        kdDebug() << "restart: sound server not running, not terminating" << endl;
        if(bVerbose)
            KPassivePopup::message(i18n("Unable to terminate sound server"),
                i18n("Sound server is not running"), this);
    }
    else if(!server.terminate())
    {
        kdDebug() << "terminate: unable to terminate sound server" << endl;
        if(bVerbose)
            KPassivePopup::message(i18n("Unable to terminate sound server"),
                i18n("Unknown reason"), this);
    }

    if(status() == -1)
        start_arts();
    else
    {
        kdDebug() << "restart: sound server running, not starting" << endl;
        if(bVerbose)
            KPassivePopup::message(i18n("Unable to start sound server"),
                i18n("Sound server is already running"), this);
    }

    repaint();
}

void kickarts::statusView()
{
    KDialogBase *dlg = new KDialogBase(KDialogBase::Plain, i18n("Sound server status"),
                     KDialogBase::Close,KDialogBase::Close, this);
    QWidget *page = dlg->plainPage();

    QBoxLayout *l= new QVBoxLayout(page);

    QLabel* statusLabel = new QLabel(page, "statusLabel");
    l->addWidget(statusLabel);

    switch(status())
    {
        case -1:
            statusLabel->setText(i18n("server status: not running"));
            break;
        case 0:
            statusLabel->setText(i18n("server status: suspended"));
            break;
        case 1:
            statusLabel->setText(i18n("server status: running, will suspend in %1 seconds").arg(server.secondsUntilSuspend()));
            break;
        case 2:
            statusLabel->setText(i18n("server status: running and busy"));
            break;
        case 3:
            statusLabel->setText(i18n("server status: running, autosuspend disabled"));
            break;
    }

    if(status()!=-1)
    {
        QLabel* rtLabel = new QLabel(page, "rtLabel");
        l->addWidget(rtLabel);
        Arts::RealtimeStatus rtStatus = server.realtimeStatus();

        switch(rtStatus)
        {
            case Arts::rtRealtime:
                rtLabel->setText(i18n("real-time status: real-time"));
                break;
            case Arts::rtNoSupport:
                rtLabel->setText(i18n("real-time status: no real-time support"));
                break;
            case Arts::rtNoWrapper:
                rtLabel->setText(i18n("real-time status: not started through real-time wrapper"));
                break;
            case Arts::rtNoRealtime:
                rtLabel->setText(i18n("real-time status: not real-time"));
                break;
            default:
                rtLabel->setText(i18n("real-time status: unknown"));
        }


         l->addWidget(new QLabel(i18n("server buffer time: %1 ms\n").arg(server.serverBufferTime()) +
                                 i18n("buffer size multiplier: %1\n").arg(server.bufferSizeMultiplier()) +
                                 i18n("minimum stream buffer time: %1 ms\n").arg(server.minStreamBufferTime()) +
                                 i18n("auto suspend time: %1 s\n").arg(server.autoSuspendSeconds()) +
                                 i18n("audio method: %1\n").arg(server.audioMethod().c_str()) +
                                 i18n("sampling rate: %1\n").arg(server.samplingRate()) +
                                 i18n("channels: %1\n").arg(server.channels()) +
                                 i18n("sample size: %1 bits\n").arg(server.bits()) +
                                 (server.fullDuplex() ? i18n("duplex: full\n") : i18n("duplex: half\n")) +
                                 i18n("device: %1\n").arg(server.audioDevice().c_str()) +
                                 i18n("fragment size: %1\n").arg(server.fragmentSize()),
                                 page));
    }

    dlg->exec();
    delete dlg;
}

void kickarts::configArts()
{
    if (configArts_process) return;
    configArts_process = new KProcess;
    *configArts_process << QString::fromLatin1("kcmshell") << QString::fromLatin1("arts");
    connect(configArts_process, SIGNAL(processExited(KProcess *)), SLOT(exitConfigArts()));
    if (!configArts_process->start())
    {
        kdDebug() << "config: couldn't start kcmshell" << endl;
        delete configArts_process;
        configArts_process = 0;
        return;
    }
}

void kickarts::exitConfigArts()
{
    delete configArts_process;
    configArts_process = 0;
}

void kickarts::help()
{
	kapp->invokeHelp();
}

void kickarts::setToolTip(const QString &text)
{
    QToolTip::add(this, text);
}

int kickarts::status()
{
    server = Arts::SoundServerV2(Arts::Reference("global:Arts_SoundServer"));

    if (server.isNull())
    {
//        kdDebug() << "status: sound server not running" << endl;
        return -1;
    }
    else
    {
        long seconds = server.secondsUntilSuspend();

        switch (seconds)
        {
        case 0:
//          kdDebug() << "status: sound server running and suspended" << endl;
          return 0;
          break;

      case -1:
//          kdDebug() << "status: sound server running and busy" << endl;
          return 2;
          break;

      case -2:
//          kdDebug() << "status: sound server running, autosuspend disabled" << endl;
          return 3;
          break;

      default:
//          kdDebug() << "status: sound server running, will suspend in " << seconds << " seconds" << endl;
          return 1;
          break;
        }
    }
}

void kickarts::start_arts()
{
    KConfig *config = new KConfig("kcmartsrc", true, false);

    config->setGroup("Arts");
    bool startRealtime = config->readBoolEntry("StartRealtime",true);
    bool x11Comm = config->readBoolEntry("X11GlobalComm",false);
    QString args = config->readEntry("Arguments","-F 5 -S 4096 -s 60 -m artsmessage -l 3 -f");

    delete config;

    // put the value of x11Comm into .mcoprc
    KConfig *X11CommConfig = new KSimpleConfig(QDir::homeDirPath()+"/.mcoprc");

    if(x11Comm)
        X11CommConfig->writeEntry("GlobalComm","Arts::X11GlobalComm");
    else
        X11CommConfig->writeEntry("GlobalComm","Arts::TmpGlobalComm");

    X11CommConfig->sync();
    delete X11CommConfig;

    kapp->kdeinitExec(startRealtime?"artswrapper":"artsd",QStringList::split(" ",args));
}


#include "kickarts.moc"
